import HttpStatus from "http-status-codes";
import {Router} from "express";
import validate from "../utils/validate";

export default class ControllerGeneric {

    constructor(service, hasCrudRoutes = true) {
        this.service = new service();
        this.schema = this.model.modelSchema;
        this.hasCrudRoutes = hasCrudRoutes;
    }

    routesOfCrud() {
        const router = new Router();

        if (this.hasCrudRoutes) {
            router.get("/", (req, res, next) => {
                this.buscarTodos(req, res, next)
            });

            router.get("/:key", (req, res, next) => {
                this.validarExistenciaModel(req, res, next)
            }, (req, res, next) => {
                this.buscarPorId(req, res, next)
            });

            router.post("/", (req, res, next) => {
                this.modelValidator(req, res, next)
            }, (req, res, next) => {
                this.inserir(req, res, next)
            });

            router.put("/:key", (req, res, next) => {
                this.modelValidator(req, res, next)
            }, (req, res, next) => {
                this.validarExistenciaModel(req, res, next)
            }, (req, res, next) => {
                this.editar(req, res, next)
            });

            router.delete("/:key", (req, res, next) => {
                this.validarExistenciaModel(req, res, next)
            }, (req, res, next) => {
                this.remover(req, res, next)
            });
        }

        return router;
    }

    routes() {
        return new Router();
    }

    buscarTodos(req, res, next) {
        this.service
            .buscarTodos()
            .then(data => res.json({data}))
            .catch(err => next(err));
    }

    buscarPorId(req, res, next) {
        this.service
            .buscarPorId(req.params.key)
            .then(data => res.json({data}))
            .catch(err => next(err));
    }

    inserir(req, res, next) {
        this.service
            .inserir(req.body)
            .then(data => res.status(HttpStatus.CREATED).json({data}))
            .catch(err => next(err));
    }

    editar(req, res, next) {
        this.service
            .editar(req.params.key, req.body)
            .then(data => res.json({data}))
            .catch(err => next(err));
    }

    remover(req, res, next) {
        this.service
            .remover(req.params.key)
            .then(data => res.status(HttpStatus.NO_CONTENT).json({data}))
            .catch(err => next(err));
    }

    modelValidator(req, res, next) {
        return validate(req.body, this.schema)
            .then(() => next())
            .catch(err => next(err));
    }

    validarExistenciaModel(req, res, next) {
        return this.service
            .buscarPorId(req.params.key)
            .then(() => next())
            .catch(err => next(err));
    }
}
